/********************************************************************************************
 *
 * CHECK_MRTGTRAF.C
 *
 * Program: MRTG (Multi-Router Traffic Grapher) traffic plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-21-1999
 *
 * Command line: CHECK_MRTGTRAF <log_file> <expire_minutes> <AVG|MAX> <iwl> <icl> <owl> <ocl>
 *
 * Description:
 *
 * This plugin will check the incoming/outgoing transfer rates of a router, switch, etc 
 * recorded in an MRTG log.  If the newest log entry is older than <expire_minutes>, a
 * WARNING status is returned.  If either the incoming or outgoing rates exceed the <icl> or
 * <ocl> thresholds (in Bytes/sec), a CRITICAL status results.  If either of the rates exceed
 * the <iwl> or <owl> thresholds (in Bytes/sec), a WARNING status results.  
 *
 * Notes:
 * - MRTG stands for the Multi Router Traffic Grapher.  It can be downloaded from
 *   http://ee-staff.ethz.ch/~oetiker/webtools/mrtg/mrtg.html
 * - While MRTG can monitor things other than traffic rates, this plugin probably won't
 *   work with much else without modification.
 * - The calculated i/o rates are a little off from what MRTG actually reports.  I'm not sure
 *   why this is right now, but will look into it for future enhancements of this plugin.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *******************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	1024


#define	TRUE		1
#define FALSE		0


int main(int argc, char **argv){
	int result;
	FILE *fp;
	int line;
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	time_t current_time;
	int expire_minutes=0;
	int use_average=TRUE;
	unsigned long incoming_warning_threshold=0L;
	unsigned long incoming_critical_threshold=0L;
	unsigned long outgoing_warning_threshold=0L;
	unsigned long outgoing_critical_threshold=0L;
	char error_message[MAX_INPUT_BUFFER];
	char log_file[MAX_INPUT_BUFFER];
	time_t timestamp=0L;
	unsigned long average_incoming_rate=0L;
	unsigned long average_outgoing_rate=0L;
	unsigned long maximum_incoming_rate=0L;
	unsigned long maximum_outgoing_rate=0L;
	unsigned long incoming_rate=0L;
	unsigned long outgoing_rate=0L;
	double adjusted_incoming_rate=0.0;
	double adjusted_outgoing_rate=0.0;
	char incoming_speed_rating[8];
	char outgoing_speed_rating[8];



	if(argc!=8){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("MRTG traffic plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-21-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <log_file> <expire_minutes> <AVG | MAX> <iwl> <icl> <owl> <ocl>\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <log_file>       = The MRTG log file for the router you want to monitor\n");
		printf(" <expire_minutes> = Minutes before MRTG data is considered to be too old\n");
		printf(" <AVG | MAX>	  = Should we check the average or maximum values in the MRTG log?\n");
		printf(" <iwl>            = Threshold value (Bytes/sec) for incoming rate to result in WARNING status\n");
		printf(" <icl>            = Threshold value (Bytes/sec) for incoming rate to result in CRITICAL status\n");
		printf(" <owl>            = Threshold value (Bytes/sec) for outgoing rate to result in WARNING status\n");
		printf(" <ocl>            = Threshold value (Bytes/sec) for outgoing rate to result in CRITICAL status\n");
		printf("\n");
		printf("This plugin will check the incoming/outgoing transfer rates from an MRTG log.\n");
		printf("If the newest log entry is older than <expire_minutes>, a WARNING status results.\n");
		printf("If either the incoming or outgoing rates exceed the <icl> or <ocl> thresholds, a CRITICAL\n");
		printf("status results.  If either of the rates exceed the <iwl> or<owl> thresholds, a WARNING\n");
		printf("status results.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the MRTG log file name */
	strcpy(log_file,argv[1]);

	/* get the number of minutes before MRTG data "expires" and generates a WARNING status */
	expire_minutes=atoi(argv[2]);

	/* see if we should use max values instead of average values */
	if(!strcmp(argv[3],"MAX"))
		use_average=FALSE;
	else
		use_average=TRUE;

	/* get the incoming warning threshold level */
	incoming_warning_threshold=strtoul(argv[4],NULL,10);

	/* get the incoming threshold level */
	incoming_critical_threshold=strtoul(argv[5],NULL,10);

	/* get the outgoing warning threshold level */
	outgoing_warning_threshold=strtoul(argv[6],NULL,10);

	/* get the critical threshold level */
	outgoing_critical_threshold=strtoul(argv[7],NULL,10);

	/* we haven't encountered any errors thus far... */
	result=STATE_OK;


	/* open the MRTG log file for reading */
	fp=fopen(log_file,"r");
	if(fp==NULL){
		printf("Unable to open MRTG log file\n");
		return STATE_UNKNOWN;
	        }

	line=0;
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		line++;

		/* skip the first line of the log file */
		if(line==1)
			continue;

		/* break out of read loop if we've passed the number of entries we want to read */
		if(line>2)
			break;

		/* grab the timestamp */
		temp_buffer=strtok(input_buffer," ");
		timestamp=strtoul(temp_buffer,NULL,10);

		/* grab the average incoming transfer rate */
		temp_buffer=strtok(NULL," ");
		average_incoming_rate=strtoul(temp_buffer,NULL,10);

		/* grab the average outgoing transfer rate */
		temp_buffer=strtok(NULL," ");
		average_outgoing_rate=strtoul(temp_buffer,NULL,10);

		/* grab the maximum incoming transfer rate */
		temp_buffer=strtok(NULL," ");
		maximum_incoming_rate=strtoul(temp_buffer,NULL,10);

		/* grab the maximum outgoing transfer rate */
		temp_buffer=strtok(NULL," ");
		maximum_outgoing_rate=strtoul(temp_buffer,NULL,10);
	        }

	/* close the log file */
	fclose(fp);

	/* if we couldn't read enough data, return an unknown error */
	if(line<=2){
		result=STATE_UNKNOWN;
		sprintf(error_message,"Unable to process MRTG log file\n");
	        }

	/* make sure the MRTG data isn't too old */
	if(result==STATE_OK){
		time(&current_time);
		if(current_time-timestamp>(expire_minutes*60)){
			result=STATE_WARNING;
			sprintf(error_message,"MRTG data has expired (%d minutes old)\n",(int)((current_time-timestamp)/60));
		        }
	        }

	/* else check the incoming/outgoing rates */
	if(result==STATE_OK){

		if(use_average==TRUE){
			incoming_rate=average_incoming_rate;
			outgoing_rate=average_outgoing_rate;
	                }
		else{
			incoming_rate=maximum_incoming_rate;
			outgoing_rate=maximum_outgoing_rate;
	                }

		/* report incoming traffic in Bytes/sec */
		if(incoming_rate<1024){
			strcpy(incoming_speed_rating,"B/s");
			adjusted_incoming_rate=(double)incoming_rate;
		        }

		/* report incoming traffic in KBytes/sec */
		else if(incoming_rate<(1024*1024)){
			strcpy(incoming_speed_rating,"KB/s");
			adjusted_incoming_rate=(double)(incoming_rate/1024.0);
		        }

		/* report incoming traffic in MBytes/sec */
		else{
			strcpy(incoming_speed_rating,"MB/s");
			adjusted_incoming_rate=(double)(incoming_rate/1024.0/1024.0);
		        }

		/* report outgoing traffic in Bytes/sec */
		if(outgoing_rate<1024){
			strcpy(outgoing_speed_rating,"B/s");
			adjusted_outgoing_rate=(double)outgoing_rate;
		        }

		/* report outgoing traffic in KBytes/sec */
		else if(outgoing_rate<(1024*1024)){
			strcpy(outgoing_speed_rating,"KB/s");
			adjusted_outgoing_rate=(double)(outgoing_rate/1024.0);
		        }

		/* report outgoing traffic in MBytes/sec */
		else{
			strcpy(outgoing_speed_rating,"MB/s");
			adjusted_outgoing_rate=(double)(outgoing_rate/1024.0/1024.0);
		        }

		if(incoming_rate>incoming_critical_threshold || outgoing_rate>outgoing_critical_threshold){
			result=STATE_CRITICAL;
			sprintf(error_message,"%s. In = %0.1f %s, %s. Out = %0.1f %s",(use_average==TRUE)?"Ave":"Max",adjusted_incoming_rate,incoming_speed_rating,(use_average==TRUE)?"Ave":"Max",adjusted_outgoing_rate,outgoing_speed_rating);
		        }
		else if(incoming_rate>incoming_warning_threshold || outgoing_rate>outgoing_warning_threshold){
			result=STATE_WARNING;
			sprintf(error_message,"%s. In = %0.1f %s, %s. Out = %0.1f %s",(use_average==TRUE)?"Ave":"Max",adjusted_incoming_rate,incoming_speed_rating,(use_average==TRUE)?"Ave":"Max",adjusted_outgoing_rate,outgoing_speed_rating);
		        }
	        }

	if(result==STATE_OK)
			printf("Traffic ok - %s. In = %0.1f %s, %s. Out = %0.1f %s\n",(use_average==TRUE)?"Ave":"Max",adjusted_incoming_rate,incoming_speed_rating,(use_average==TRUE)?"Ave":"Max",adjusted_outgoing_rate,outgoing_speed_rating);
	else
		printf("%s\n",error_message);
		

	return result;
        }






