/******************************************************************************************
 *
 * CHECK_PING.C
 *
 * Program: Ping plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 08-14-1999
 *
 * Command line: CHECK_PING <host_address> <wpl> <cpl> <wrta> <crta> [packets]
 *
 * Description:
 *
 * This plugin will use the /bin/ping command to ping the specified host.  If the packet
 * loss or round trip average is above the <cpl> or <crta> level, a STATE_CRITICAL is
 * returned.  If the values are above the <wpl> or <wrta> levels, a STATE_WARNING is
 * returned.  Specifying an optional [packets] argument will adjust the default number
 * of ICMP ECHO packets that the plugin will send to the host.  The default number of
 * packets is 5.
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	256

#define DEFAULT_MAX_PACKETS	5	/* default number of ICMP ECHO packets */

int my_system(char *);



int main(int argc, char **argv){
	char server_name[256];
	char command_line[256];
	int wpl;
	int cpl;
	int pl=0;
	float wrta;
	float crta;
	float rta=0.0;
	int result=STATE_OK;
	int max_packets=DEFAULT_MAX_PACKETS;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;

	if(argc<6||argc>7){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("Ping plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 08-14-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> <wpl> <cpl> <wrta> <crta> [packets]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <wpl>     = Percent of packet loss necessary to result in a WARNING state\n");
		printf(" <cpl>     = Percent of packet loss necessary to result in a CRITICAL state\n");
		printf(" <wrta>    = Round trip average in milliseconds necessary to result in a WARNING state\n");
		printf(" <crta>    = Round trip average in milliseconds necessary to result in a CRITICAL state\n");
		printf(" [packets] = Optional number of ICMP ECHO packets to send out.  Default is 5 packets.\n");
		printf("\n");
		printf("This plugin will use the /bin/ping command to ping the specified host and calculate\n");
		printf("percent packet loss and round trip average.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the server address */
	strcpy(server_name,argv[1]);

	/* get the percent loss variables */
	wpl=atoi(argv[2]);
	cpl=atoi(argv[3]);

	/* get the round trip average variables */
	wrta=atof(argv[4]);
	crta=atof(argv[5]);

	/* get the number of pings */
	if(argc==7)
		max_packets=atoi(argv[6]);
	else
		max_packets=DEFAULT_MAX_PACKETS;

	/* make sure the number of packets is valid */
	if(max_packets<=0)
	        max_packets=DEFAULT_MAX_PACKETS;

	/* run the command */
	sprintf(command_line,PING_COMMAND,server_name,max_packets);
	fp = popen(command_line,"r");
	if(fp==NULL){
	  printf("Unable to open pipe: %s",command_line);
	  return STATE_UNKNOWN;
	}

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		/* get the percent loss statistics */
		if(strstr(input_buffer,"packets transmitted")){
			temp_buffer=strtok(input_buffer,",");
			temp_buffer=strtok(NULL,",");
			temp_buffer=strtok(NULL,"%");
			pl=atoi(temp_buffer);
			if(pl>=cpl){
				result=STATE_CRITICAL;
				break;
			        }
			else if(pl>=wpl){
				result=STATE_WARNING;
				break;
			        }
		        }

		/* get the round trip average */
		else if(strstr(input_buffer,"round-trip")){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"/");
			temp_buffer=strtok(NULL,"/");
			rta=atof(temp_buffer);
			if(rta>=crta){
				result=STATE_CRITICAL;
				break;
				}
			else if(rta>=wrta){
				result=STATE_WARNING;
				break;
			        }
		        }
	        }

	/* close the temp file */
	pclose(fp);
	
	if(result==STATE_OK)
		printf("PING ok - Packet loss = %d%%, RTA = %2.2f ms\n",pl,rta);
	else if(result==STATE_UNKNOWN)
		printf("Unable to read output\n");
	else
		printf("Packet loss = %d%%, RTA = %2.2f ms\n",pl,rta);

	return result;
        }



int my_system(char *command){
        int pid;
	int status;
	int result;
	extern int errno;
	extern char **environ;


	/* if no command was passed, return with no error */
	if(command==NULL)
	        return STATE_OK;

	/* fork */
	pid=fork();

	/* return an error if we couldn't fork */
	if(pid==-1)
	        return STATE_UNKNOWN;  

	/* execute the command in the child process */
        if (pid==0){
		char *argv[4];
		argv[0]="sh";
		argv[1]="-c";
		argv[2]=command;
		argv[3]=0;
		execve("/bin/sh",argv,environ);
		exit(STATE_UNKNOWN);
	        }

	/* wait for the command to finish */
        do{
                if (waitpid(pid, &status, 0) == -1) {
		        if (errno != EINTR)
                                return STATE_UNKNOWN;
                        } 

		/* get the exit code returned from the program */
		result=WEXITSTATUS(status);

		/* check the bounds for the return code */
		if(result<0||result>2)
			return STATE_UNKNOWN;

		return result;
           } while(1);

        return STATE_OK;
        }

